/*!
FullCalendar Resource Time Grid Plugin v7.0.0-rc.0
Docs & License: https://fullcalendar.io/docs/vertical-resource-view
(c) 2024 Adam Shaw
*/
FullCalendar.ResourceTimeGrid = (function (exports, core, premiumCommonPlugin, resourcePlugin, timeGridPlugin, internal$2, preact, internal$4, internal$3, internal$1, internal$5) {
    'use strict';

    function _interopDefault (e) { return e && e.__esModule ? e : { 'default': e }; }

    var premiumCommonPlugin__default = /*#__PURE__*/_interopDefault(premiumCommonPlugin);
    var resourcePlugin__default = /*#__PURE__*/_interopDefault(resourcePlugin);
    var timeGridPlugin__default = /*#__PURE__*/_interopDefault(timeGridPlugin);

    class ResourceDayTimeColsJoiner extends internal$1.VResourceJoiner {
        transformSeg(seg, resourceDayTable, resourceI) {
            return [
                Object.assign(Object.assign({}, seg), { col: resourceDayTable.computeCol(seg.col, resourceI) }),
            ];
        }
    }

    class ResourceTimeGridView extends internal$2.DateComponent {
        constructor() {
            super(...arguments);
            // memo
            this.flattenResources = internal$2.memoize(internal$1.flattenResources);
            this.buildResourceTimeColsModel = internal$2.memoize(buildResourceTimeColsModel);
            this.buildResourceRowConfigs = internal$2.memoize(internal$3.buildResourceRowConfigs);
            this.createDayHeaderFormatter = internal$2.memoize(internal$4.createDayHeaderFormatter);
            // internal
            this.allDaySplitter = new internal$5.AllDaySplitter();
            // for all-day-resource props
            this.allDayResourceSplitter = new internal$1.VResourceSplitter();
            this.allDayResourceSlicers = {};
            this.allDayResourceJoiner = new internal$3.ResourceDayTableJoiner();
            // for timed resource props
            this.buildDayRanges = internal$2.memoize(internal$5.buildDayRanges);
            this.timedResourceSplitter = new internal$1.VResourceSplitter();
            this.timedResourceSlicers = {};
            this.timedResourceJoiner = new ResourceDayTimeColsJoiner();
            // timed-only column splitting
            this.splitFgEventSegs = internal$2.memoize(internal$5.organizeSegsByCol);
            this.splitBgEventSegs = internal$2.memoize(internal$5.organizeSegsByCol);
            this.splitBusinessHourSegs = internal$2.memoize(internal$5.organizeSegsByCol);
            this.splitNowIndicatorSegs = internal$2.memoize(internal$5.organizeSegsByCol);
            this.splitDateSelectionSegs = internal$2.memoize(internal$5.organizeSegsByCol);
            this.splitEventDrag = internal$2.memoize(internal$5.splitInteractionByCol);
            this.splitEventResize = internal$2.memoize(internal$5.splitInteractionByCol);
            this.isHitComboAllowed = (hit0, hit1) => {
                let allowAcrossResources = this.dayRanges.length === 1;
                return allowAcrossResources || hit0.dateSpan.resourceId === hit1.dateSpan.resourceId;
            };
        }
        render() {
            let { props, context } = this;
            let { options, dateEnv } = context;
            let { dateProfile } = props;
            let resourceOrderSpecs = options.resourceOrder || internal$1.DEFAULT_RESOURCE_ORDER;
            let resources = this.flattenResources(props.resourceStore, resourceOrderSpecs);
            let resourceDayTableModel = this.buildResourceTimeColsModel(dateProfile, context.dateProfileGenerator, resources, options.datesAboveResources, context);
            // split seg by all-day/timed
            let splitProps = this.allDaySplitter.splitProps(props);
            // split the all-day segs by resource
            let allDayResourceSplitProps = this.allDayResourceSplitter.splitProps({
                businessHours: splitProps.allDay.businessHours,
                dateSelection: splitProps.allDay.dateSelection,
                eventStore: splitProps.allDay.eventStore,
                eventUiBases: splitProps.allDay.eventUiBases,
                eventSelection: splitProps.allDay.eventSelection,
                eventDrag: splitProps.allDay.eventDrag,
                eventResize: splitProps.allDay.eventResize,
                resourceDayTableModel,
            });
            this.allDayResourceSlicers = internal$2.mapHash(allDayResourceSplitProps, (split, resourceId) => this.allDayResourceSlicers[resourceId] || new internal$4.DayTableSlicer());
            let allDayResourceSlicedProps = internal$2.mapHash(this.allDayResourceSlicers, (slicer, resourceId) => slicer.sliceProps(allDayResourceSplitProps[resourceId], dateProfile, options.nextDayThreshold, context, resourceDayTableModel.dayTableModel));
            let allDayResourceJoinedProps = this.allDayResourceJoiner.joinProps(allDayResourceSlicedProps, resourceDayTableModel);
            // split the timed segs by resource
            let dayRanges = this.dayRanges = this.buildDayRanges(resourceDayTableModel.dayTableModel, dateProfile, dateEnv);
            let timedResourceSplitProps = this.timedResourceSplitter.splitProps({
                businessHours: splitProps.timed.businessHours,
                dateSelection: splitProps.timed.dateSelection,
                eventStore: splitProps.timed.eventStore,
                eventUiBases: splitProps.timed.eventUiBases,
                eventSelection: splitProps.timed.eventSelection,
                eventDrag: splitProps.timed.eventDrag,
                eventResize: splitProps.timed.eventResize,
                resourceDayTableModel,
            });
            this.timedResourceSlicers = internal$2.mapHash(timedResourceSplitProps, (split, resourceId) => this.timedResourceSlicers[resourceId] || new internal$5.DayTimeColsSlicer());
            let timedResourceSlicedProps = internal$2.mapHash(this.timedResourceSlicers, (slicer, resourceId) => slicer.sliceProps(timedResourceSplitProps[resourceId], dateProfile, null, context, dayRanges));
            let timedResourceJoinedProps = this.timedResourceJoiner.joinProps(timedResourceSlicedProps, resourceDayTableModel);
            let datesRepDistinctDays = resourceDayTableModel.dayTableModel.rowCnt === 1;
            let dayHeaderFormat = this.createDayHeaderFormatter(context.options.dayHeaderFormat, datesRepDistinctDays, resourceDayTableModel.colCnt);
            return (preact.createElement(internal$2.NowTimer, { unit: options.nowIndicator ? 'minute' : 'day' /* hacky */ }, (nowDate, todayRange) => {
                // timed-only column splitting
                let colCnt = resourceDayTableModel.colCnt;
                let fgEventSegsByCol = this.splitFgEventSegs(timedResourceJoinedProps.fgEventSegs, colCnt);
                let bgEventSegsByCol = this.splitBgEventSegs(timedResourceJoinedProps.bgEventSegs, colCnt);
                let businessHourSegsByCol = this.splitBusinessHourSegs(timedResourceJoinedProps.businessHourSegs, colCnt);
                let nowIndicatorSegsByCol = this.splitNowIndicatorSegs((() => {
                    // was buildNowIndicatorSegs
                    let nonResourceSegs = options.nowIndicator
                        ? this.timedResourceSlicers[''].sliceNowDate(nowDate, this.props.dateProfile, this.context.options.nextDayThreshold, this.context, this.dayRanges)
                        : []; // TODO: breaks memoization?
                    return this.timedResourceJoiner.expandSegs(resourceDayTableModel, nonResourceSegs);
                })(), colCnt);
                let dateSelectionSegsByCol = this.splitDateSelectionSegs(timedResourceJoinedProps.dateSelectionSegs, colCnt);
                let eventDragByCol = this.splitEventDrag(timedResourceJoinedProps.eventDrag, colCnt);
                let eventResizeByCol = this.splitEventResize(timedResourceJoinedProps.eventResize, colCnt);
                const headerTiers = this.buildResourceRowConfigs(resources, options.datesAboveResources, resourceDayTableModel.dayTableModel.headerDates, datesRepDistinctDays, props.dateProfile, todayRange, dayHeaderFormat, context);
                return (preact.createElement(internal$5.TimeGridLayout, { labelId: props.labelId, labelStr: props.labelStr, dateProfile: dateProfile, nowDate: nowDate, todayRange: todayRange, cells: resourceDayTableModel.cells[0], forPrint: props.forPrint, isHitComboAllowed: this.isHitComboAllowed, className: 'fc-resource-timegrid fc-timegrid', 
                    // header content
                    headerTiers: headerTiers, 
                    // all-day content
                    fgEventSegs: allDayResourceJoinedProps.fgEventSegs, bgEventSegs: allDayResourceJoinedProps.bgEventSegs, businessHourSegs: allDayResourceJoinedProps.businessHourSegs, dateSelectionSegs: allDayResourceJoinedProps.dateSelectionSegs, eventDrag: allDayResourceJoinedProps.eventDrag, eventResize: allDayResourceJoinedProps.eventResize, 
                    // timed content
                    fgEventSegsByCol: fgEventSegsByCol, bgEventSegsByCol: bgEventSegsByCol, businessHourSegsByCol: businessHourSegsByCol, nowIndicatorSegsByCol: nowIndicatorSegsByCol, dateSelectionSegsByCol: dateSelectionSegsByCol, eventDragByCol: eventDragByCol, eventResizeByCol: eventResizeByCol, 
                    // universal content
                    eventSelection: allDayResourceJoinedProps.eventSelection || splitProps.timed.eventSelection }));
            }));
        }
    }
    /*
    TODO: kill this and DayResourceTableModel/ResourceDayTableModel
    */
    function buildResourceTimeColsModel(dateProfile, dateProfileGenerator, resources, datesAboveResources, context) {
        let dayTable = internal$5.buildTimeColsModel(dateProfile, dateProfileGenerator);
        if (!resources.length) {
            return new internal$1.ResourcelessDayTableModel(dayTable);
        }
        return datesAboveResources ?
            new internal$1.DayResourceTableModel(dayTable, resources, context) :
            new internal$1.ResourceDayTableModel(dayTable, resources, context);
    }

    var plugin = core.createPlugin({
        name: '@fullcalendar/resource-timegrid',
        premiumReleaseDate: '2025-02-21',
        deps: [
            premiumCommonPlugin__default["default"],
            resourcePlugin__default["default"],
            timeGridPlugin__default["default"],
        ],
        initialView: 'resourceTimeGridDay',
        views: {
            resourceTimeGrid: {
                type: 'timeGrid',
                component: ResourceTimeGridView,
                needsResourceData: true,
            },
            resourceTimeGridDay: {
                type: 'resourceTimeGrid',
                duration: { days: 1 },
            },
            resourceTimeGridWeek: {
                type: 'resourceTimeGrid',
                duration: { weeks: 1 },
            },
        },
    });

    var internal = {
        __proto__: null,
        ResourceTimeGridView: ResourceTimeGridView
    };

    core.globalPlugins.push(plugin);

    exports.Internal = internal;
    exports["default"] = plugin;

    Object.defineProperty(exports, '__esModule', { value: true });

    return exports;

})({}, FullCalendar, FullCalendar.PremiumCommon, FullCalendar.Resource, FullCalendar.TimeGrid, FullCalendar.Internal, FullCalendar.Preact, FullCalendar.DayGrid.Internal, FullCalendar.ResourceDayGrid.Internal, FullCalendar.Resource.Internal, FullCalendar.TimeGrid.Internal);
