'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var internal_cjs$1 = require('@fullcalendar/core/internal.cjs');
var preact_cjs = require('@fullcalendar/core/preact.cjs');
var internal_cjs$3 = require('@fullcalendar/daygrid/internal.cjs');
var internal_cjs$2 = require('@fullcalendar/resource-daygrid/internal.cjs');
var internal_cjs = require('@fullcalendar/resource/internal.cjs');
var internal_cjs$4 = require('@fullcalendar/timegrid/internal.cjs');

class ResourceDayTimeColsJoiner extends internal_cjs.VResourceJoiner {
    transformSeg(seg, resourceDayTable, resourceI) {
        return [
            Object.assign(Object.assign({}, seg), { col: resourceDayTable.computeCol(seg.col, resourceI) }),
        ];
    }
}

class ResourceTimeGridView extends internal_cjs$1.DateComponent {
    constructor() {
        super(...arguments);
        // memo
        this.flattenResources = internal_cjs$1.memoize(internal_cjs.flattenResources);
        this.buildResourceTimeColsModel = internal_cjs$1.memoize(buildResourceTimeColsModel);
        this.buildResourceRowConfigs = internal_cjs$1.memoize(internal_cjs$2.buildResourceRowConfigs);
        this.createDayHeaderFormatter = internal_cjs$1.memoize(internal_cjs$3.createDayHeaderFormatter);
        // internal
        this.allDaySplitter = new internal_cjs$4.AllDaySplitter();
        // for all-day-resource props
        this.allDayResourceSplitter = new internal_cjs.VResourceSplitter();
        this.allDayResourceSlicers = {};
        this.allDayResourceJoiner = new internal_cjs$2.ResourceDayTableJoiner();
        // for timed resource props
        this.buildDayRanges = internal_cjs$1.memoize(internal_cjs$4.buildDayRanges);
        this.timedResourceSplitter = new internal_cjs.VResourceSplitter();
        this.timedResourceSlicers = {};
        this.timedResourceJoiner = new ResourceDayTimeColsJoiner();
        // timed-only column splitting
        this.splitFgEventSegs = internal_cjs$1.memoize(internal_cjs$4.organizeSegsByCol);
        this.splitBgEventSegs = internal_cjs$1.memoize(internal_cjs$4.organizeSegsByCol);
        this.splitBusinessHourSegs = internal_cjs$1.memoize(internal_cjs$4.organizeSegsByCol);
        this.splitNowIndicatorSegs = internal_cjs$1.memoize(internal_cjs$4.organizeSegsByCol);
        this.splitDateSelectionSegs = internal_cjs$1.memoize(internal_cjs$4.organizeSegsByCol);
        this.splitEventDrag = internal_cjs$1.memoize(internal_cjs$4.splitInteractionByCol);
        this.splitEventResize = internal_cjs$1.memoize(internal_cjs$4.splitInteractionByCol);
        this.isHitComboAllowed = (hit0, hit1) => {
            let allowAcrossResources = this.dayRanges.length === 1;
            return allowAcrossResources || hit0.dateSpan.resourceId === hit1.dateSpan.resourceId;
        };
    }
    render() {
        let { props, context } = this;
        let { options, dateEnv } = context;
        let { dateProfile } = props;
        let resourceOrderSpecs = options.resourceOrder || internal_cjs.DEFAULT_RESOURCE_ORDER;
        let resources = this.flattenResources(props.resourceStore, resourceOrderSpecs);
        let resourceDayTableModel = this.buildResourceTimeColsModel(dateProfile, context.dateProfileGenerator, resources, options.datesAboveResources, context);
        // split seg by all-day/timed
        let splitProps = this.allDaySplitter.splitProps(props);
        // split the all-day segs by resource
        let allDayResourceSplitProps = this.allDayResourceSplitter.splitProps({
            businessHours: splitProps.allDay.businessHours,
            dateSelection: splitProps.allDay.dateSelection,
            eventStore: splitProps.allDay.eventStore,
            eventUiBases: splitProps.allDay.eventUiBases,
            eventSelection: splitProps.allDay.eventSelection,
            eventDrag: splitProps.allDay.eventDrag,
            eventResize: splitProps.allDay.eventResize,
            resourceDayTableModel,
        });
        this.allDayResourceSlicers = internal_cjs$1.mapHash(allDayResourceSplitProps, (split, resourceId) => this.allDayResourceSlicers[resourceId] || new internal_cjs$3.DayTableSlicer());
        let allDayResourceSlicedProps = internal_cjs$1.mapHash(this.allDayResourceSlicers, (slicer, resourceId) => slicer.sliceProps(allDayResourceSplitProps[resourceId], dateProfile, options.nextDayThreshold, context, resourceDayTableModel.dayTableModel));
        let allDayResourceJoinedProps = this.allDayResourceJoiner.joinProps(allDayResourceSlicedProps, resourceDayTableModel);
        // split the timed segs by resource
        let dayRanges = this.dayRanges = this.buildDayRanges(resourceDayTableModel.dayTableModel, dateProfile, dateEnv);
        let timedResourceSplitProps = this.timedResourceSplitter.splitProps({
            businessHours: splitProps.timed.businessHours,
            dateSelection: splitProps.timed.dateSelection,
            eventStore: splitProps.timed.eventStore,
            eventUiBases: splitProps.timed.eventUiBases,
            eventSelection: splitProps.timed.eventSelection,
            eventDrag: splitProps.timed.eventDrag,
            eventResize: splitProps.timed.eventResize,
            resourceDayTableModel,
        });
        this.timedResourceSlicers = internal_cjs$1.mapHash(timedResourceSplitProps, (split, resourceId) => this.timedResourceSlicers[resourceId] || new internal_cjs$4.DayTimeColsSlicer());
        let timedResourceSlicedProps = internal_cjs$1.mapHash(this.timedResourceSlicers, (slicer, resourceId) => slicer.sliceProps(timedResourceSplitProps[resourceId], dateProfile, null, context, dayRanges));
        let timedResourceJoinedProps = this.timedResourceJoiner.joinProps(timedResourceSlicedProps, resourceDayTableModel);
        let datesRepDistinctDays = resourceDayTableModel.dayTableModel.rowCnt === 1;
        let dayHeaderFormat = this.createDayHeaderFormatter(context.options.dayHeaderFormat, datesRepDistinctDays, resourceDayTableModel.colCnt);
        return (preact_cjs.createElement(internal_cjs$1.NowTimer, { unit: options.nowIndicator ? 'minute' : 'day' /* hacky */ }, (nowDate, todayRange) => {
            // timed-only column splitting
            let colCnt = resourceDayTableModel.colCnt;
            let fgEventSegsByCol = this.splitFgEventSegs(timedResourceJoinedProps.fgEventSegs, colCnt);
            let bgEventSegsByCol = this.splitBgEventSegs(timedResourceJoinedProps.bgEventSegs, colCnt);
            let businessHourSegsByCol = this.splitBusinessHourSegs(timedResourceJoinedProps.businessHourSegs, colCnt);
            let nowIndicatorSegsByCol = this.splitNowIndicatorSegs((() => {
                // was buildNowIndicatorSegs
                let nonResourceSegs = options.nowIndicator
                    ? this.timedResourceSlicers[''].sliceNowDate(nowDate, this.props.dateProfile, this.context.options.nextDayThreshold, this.context, this.dayRanges)
                    : []; // TODO: breaks memoization?
                return this.timedResourceJoiner.expandSegs(resourceDayTableModel, nonResourceSegs);
            })(), colCnt);
            let dateSelectionSegsByCol = this.splitDateSelectionSegs(timedResourceJoinedProps.dateSelectionSegs, colCnt);
            let eventDragByCol = this.splitEventDrag(timedResourceJoinedProps.eventDrag, colCnt);
            let eventResizeByCol = this.splitEventResize(timedResourceJoinedProps.eventResize, colCnt);
            const headerTiers = this.buildResourceRowConfigs(resources, options.datesAboveResources, resourceDayTableModel.dayTableModel.headerDates, datesRepDistinctDays, props.dateProfile, todayRange, dayHeaderFormat, context);
            return (preact_cjs.createElement(internal_cjs$4.TimeGridLayout, { labelId: props.labelId, labelStr: props.labelStr, dateProfile: dateProfile, nowDate: nowDate, todayRange: todayRange, cells: resourceDayTableModel.cells[0], forPrint: props.forPrint, isHitComboAllowed: this.isHitComboAllowed, className: 'fc-resource-timegrid fc-timegrid', 
                // header content
                headerTiers: headerTiers, 
                // all-day content
                fgEventSegs: allDayResourceJoinedProps.fgEventSegs, bgEventSegs: allDayResourceJoinedProps.bgEventSegs, businessHourSegs: allDayResourceJoinedProps.businessHourSegs, dateSelectionSegs: allDayResourceJoinedProps.dateSelectionSegs, eventDrag: allDayResourceJoinedProps.eventDrag, eventResize: allDayResourceJoinedProps.eventResize, 
                // timed content
                fgEventSegsByCol: fgEventSegsByCol, bgEventSegsByCol: bgEventSegsByCol, businessHourSegsByCol: businessHourSegsByCol, nowIndicatorSegsByCol: nowIndicatorSegsByCol, dateSelectionSegsByCol: dateSelectionSegsByCol, eventDragByCol: eventDragByCol, eventResizeByCol: eventResizeByCol, 
                // universal content
                eventSelection: allDayResourceJoinedProps.eventSelection || splitProps.timed.eventSelection }));
        }));
    }
}
/*
TODO: kill this and DayResourceTableModel/ResourceDayTableModel
*/
function buildResourceTimeColsModel(dateProfile, dateProfileGenerator, resources, datesAboveResources, context) {
    let dayTable = internal_cjs$4.buildTimeColsModel(dateProfile, dateProfileGenerator);
    if (!resources.length) {
        return new internal_cjs.ResourcelessDayTableModel(dayTable);
    }
    return datesAboveResources ?
        new internal_cjs.DayResourceTableModel(dayTable, resources, context) :
        new internal_cjs.ResourceDayTableModel(dayTable, resources, context);
}

exports.ResourceTimeGridView = ResourceTimeGridView;
